import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus, QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DatabaseException } from 'src/modules/database/exceptions/database.exception';
import { ICollection } from 'src/modules/database/interfaces/collection.interface';
import { RpcValidationPipe } from 'src/utils/pipes/rpc.validation-pipe';
import { CreateConfigurationCommand } from '../../commands/create-configuration.command';
import { DeleteConfigurationCommand } from '../../commands/delete-configuration.command';
import { UpdateConfigurationCommand } from '../../commands/update-configuration.command';
import { CreateConfigurationRequest } from '../../domain/dtos/create-configuration.request';
import { FindAllConfigurationsRequest } from '../../domain/dtos/find-all-configurations.request';
import { FindConfigurationByUuidRequest } from '../../domain/dtos/find-configuration-by-uuid.request';
import { UpdateConfigurationRequest } from '../../domain/dtos/update-configuration.request';
import { Configuration } from '../../domain/entities/configuration';
import { FindAllConfigurationsQuery } from '../../queries/find-all-configurations.query';
import { FindConfigurationByUuidQuery } from '../../queries/find-configuration-by-uuid.query';
import { PropagateConfigurationsQuery } from '../../queries/propagate-configurations.query';
import { ConfigurationPresenter } from './configuration.presenter';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class ConfigurationController {
  constructor(
    private readonly commandBus: CommandBus,
    private readonly queryBus: QueryBus,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  @GrpcMethod('ConfigurationService', 'FindAll')
  async findAll(
    data: FindAllConfigurationsRequest,
  ): Promise<ICollection<Configuration>> {
    const configurationCollection = await this.queryBus.execute(
      new FindAllConfigurationsQuery(data),
    );
    return Promise.resolve({
      data: configurationCollection.data.map((configuration: Configuration) =>
        this.mapper.map(configuration, Configuration, ConfigurationPresenter),
      ),
      total: configurationCollection.total,
    });
  }

  @GrpcMethod('ConfigurationService', 'FindOneByUuid')
  async findOneByUuid(
    data: FindConfigurationByUuidRequest,
  ): Promise<ConfigurationPresenter> {
    try {
      const configuration = await this.queryBus.execute(
        new FindConfigurationByUuidQuery(data),
      );
      return this.mapper.map(
        configuration,
        Configuration,
        ConfigurationPresenter,
      );
    } catch (error) {
      throw new RpcException({
        code: 5,
        message: 'Configuration not found',
      });
    }
  }

  @GrpcMethod('ConfigurationService', 'Create')
  async createConfiguration(
    data: CreateConfigurationRequest,
  ): Promise<ConfigurationPresenter> {
    try {
      const configuration = await this.commandBus.execute(
        new CreateConfigurationCommand(data),
      );
      return this.mapper.map(
        configuration,
        Configuration,
        ConfigurationPresenter,
      );
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('Already exists')) {
          throw new RpcException({
            code: 6,
            message: 'Configuration already exists',
          });
        }
      }
      throw new RpcException({});
    }
  }

  @GrpcMethod('ConfigurationService', 'Update')
  async updateConfiguration(
    data: UpdateConfigurationRequest,
  ): Promise<ConfigurationPresenter> {
    try {
      const configuration = await this.commandBus.execute(
        new UpdateConfigurationCommand(data),
      );

      return this.mapper.map(
        configuration,
        Configuration,
        ConfigurationPresenter,
      );
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('not found')) {
          throw new RpcException({
            code: 5,
            message: 'Configuration not found',
          });
        }
      }
      throw new RpcException({});
    }
  }

  @GrpcMethod('ConfigurationService', 'Delete')
  async deleteConfiguration(
    data: FindConfigurationByUuidRequest,
  ): Promise<void> {
    try {
      await this.commandBus.execute(new DeleteConfigurationCommand(data.uuid));

      return Promise.resolve();
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('not found')) {
          throw new RpcException({
            code: 5,
            message: 'Configuration not found',
          });
        }
      }
      throw new RpcException({});
    }
  }

  @GrpcMethod('ConfigurationService', 'Propagate')
  async propagate(): Promise<void> {
    try {
      await this.queryBus.execute(new PropagateConfigurationsQuery());
    } catch (e) {
      throw new RpcException({});
    }
  }
}
